from setuptools import setup, find_packages

with open('requirements.txt') as f:
    required = f.read().splitlines()
setup(
    name="AIPyS",
    version="0.1.3",
    packages=find_packages(include=['AIPyS','AIPyS.CLI','AIPyS.classification.bayes','AIPyS.classification.CNN',
                                    'AIPyS.segmentation.cellpose','AIPyS.segmentation.parametric','AIPyS.supportFunctions',
                                    'web_app.Image_labeling','web_app.measure_length','web_app.measure_length','web_app.TableViz', 'web_app.AreasViz',]),
    install_requires=required,
    entry_points={
        'console_scripts': [
           'aipys=AIPyS.CLI.aipys:main',  
           'updateParameters=AIPyS.CLI.set_parameters:main',
            'load-parameters=AIPyS.CLI.loadParameters:main', 
            ],
    },
    author="Gil Kanfer",
    author_email="gil.kanfer.il@gmail.com",
    description="AI Powered Photoswitchable Screen",
    long_description=open('README.md').read(),
    long_description_content_type="text/markdown",
    url="",  # Consider adding a URL if you have a project page or source repository
    classifiers=[
        "Programming Language :: Python :: 3",
        "License :: OSI Approved :: MIT License",
        "Operating System :: OS Independent",
    ],
    python_requires='>=3.8',  # Specify the minimum version of Python required
)